let lastDailyTCVStats = [];
let lastSaleDistribution = [];
let lastDailyStats = [];
let lastMonthlyStats = [];

function resetLastStatsVariables() {
  lastDailyTCVStats = [];
  lastSaleDistribution = [];
  lastDailyStats = [];
  lastMonthlyStats = [];
}

function initializeStats(data) {
  if (!data.hasOwnProperty('proStats')) {
    return;
  }
  let dailyStats = data.proStats.dailyStats.reverse();
  let monthlyStats = data.proStats.monthlyStats.reverse();

  let dailyStatsUnion = _.intersectionWith(dailyStats, lastDailyStats, _.isEqual);
  let monthlyStatsUnion = _.intersectionWith(monthlyStats, lastMonthlyStats, _.isEqual);
  let saleDistributionUnion = _.intersectionWith(data.proStats.saleDistribution, lastSaleDistribution, _.isEqual);
  let dailyTCVStatsUnion = _.intersectionWith(data.proStats.dailyTCVStats, lastDailyTCVStats, _.isEqual);

  if (dailyStatsUnion.length !== dailyStats.length) {
    prepareProfitCharts(dailyStats, 'dailyProfitsChart');
    prepareEarnedVsSpentCharts(dailyStats, 'dailySpentEarnedChart');
    prepareBuysVsSalesCharts(dailyStats, 'dailyBuySalesChart');
    lastDailyStats = dailyStats;
  }

  if (monthlyStatsUnion.length !== monthlyStats.length) {
    prepareProfitCharts(monthlyStats, 'monthlyProfitsChart');
    prepareEarnedVsSpentCharts(monthlyStats, 'monthlySpentEarnedChart');
    prepareBuysVsSalesCharts(monthlyStats, 'monthlyBuySalesChart');
    lastMonthlyStats = monthlyStats;
  }

  if (saleDistributionUnion.length !== data.proStats.saleDistribution.length) {
    prepareSaleDistributionChart(data.proStats.saleDistribution, 'saleDistribution');
    lastSaleDistribution = data.proStats.saleDistribution;
  }

  if (dailyTCVStatsUnion.length !== data.proStats.dailyTCVStats.length) {
    prepareTCVChart(data.proStats.dailyTCVStats, 'TCVChart');
    lastDailyTCVStats = data.proStats.dailyTCVStats;
  }
}

function prepareTCVChart(data, element) {
    let labels = []
    let minTCV = []
    let maxTCV = []

    let sortedData = _.sortBy(data, [function(o) { return o.date; }]);

    for (i = 0; i < sortedData.length; i++) {
        labels.push(sortedData[i].date.replace("-", "/").replace("-", "/"))

        minTCV.push(parseFloat((sortedData[i].minTCV).toFixed(8)))
        maxTCV.push(parseFloat((sortedData[i].maxTCV).toFixed(8)))
    }

    new Chart(document.getElementById(element),
        {
            "type":"line",
            "data": {
                "labels": labels,
                "datasets": [
                    {
                        "label": i18next.t('statsPage.minTCV'),
                        "data": minTCV,
                        "fill": false,
                        "borderColor":"rgb(75, 192, 192)",
                        "pointRadius": 3,
                        "lineTension":0.4
                    },
                    {
                        "label": i18next.t('statsPage.maxTCV'),
                        "data": maxTCV,
                        "fill": false,
                        "borderColor":"#ef5350",
                        "pointRadius": 3,
                        "lineTension":0.4
                    }
                ],
            },
            "options":{
                "responsive": true,
                "maintainAspectRatio": false,
                "legend": {
                    "display": true,
                },
                "animation": false,
                "tooltips": {
                    "mode": "index",
                    "intersect": false,
                },
                "hover": {
                    "mode": "index",
                    "intersect": true,
                },
                "scales": {
                    "yAxes": [{
                        "ticks": {
                            "fontColor":"rgba(152,166,173,0.5)",
                            "beginAtZero": true,
                            callback: function(value, index, values) {
                                return value.toFixed(8).toString()
                            }
                        },
                        "gridLines": {
                            "display": true,
                            "color": "rgba(152,166,173,0.1)",
                            "zeroLineColor": "rgba(152,166,173,0.1)",
                        }
                    }],
                    "xAxes": [{
                        "ticks": {
                            "fontColor":"rgba(152,166,173,0.5)",
                            "maxTicksLimit": 10,

                        },
                        "gridLines": {
                            "display": true,
                            "color": "rgba(152,166,173,0.1)",
                        }
                    }]
                }
            }
        });
}

function prepareSaleDistributionChart(data, element) {
    let labels = [], values = []
    data.forEach(element => {
        labels.push(element.name)
        values.push(element.value)
    });

    new Chart(document.getElementById(element),
        {
            "type":"doughnut",
            "data": {
                "labels": labels,
                "datasets":[
                    {
                        "label": i18next.t('statsPage.SalesDistribution'),
                        "data": values,
                        "backgroundColor":[
                            "#FF9900",
                            "#3bafda",
                            "#00b19d",
                            "#ef5350",
                            "#ef5350"
                        ]
                    }
                ]
            },
            "options": {
                "maintainAspectRatio": false,
                "plugins": {
                    "labels": [
                        {
                            "render": 'percentage',
                            "fontColor": "#000",
                            "precision": 0,
                            "showZero": false,
                        }
                    ]
                },
                "legend": {
                    "display": true,
                },
                "animation": false
            }
        });
}

function prepareBuysVsSalesCharts(sales, element) {
    let labels = []
    let sells = []
    let buys = []

    for (i = 0; i < sales.length; i++) {
        let date
        if(element === "monthlyBuySalesChart") {
            date = sales[i].month.replace("-", "/")
        } else {
            date = sales[i].date.replace("-", "/").replace("-", "/")
        }

        labels.push(date)

        sells.push((sales[i].totalSales))
        buys.push((sales[i].totalBuys))
    }

    new Chart(document.getElementById(element),
        {
            "type":"bar",
            "data": {
                "labels": labels,
                "datasets":[
                    {
                        "label": i18next.t('statsPage.Sales'),
                        "data": sells,
                        "backgroundColor":"rgb(75, 192, 192)",
                    },
                    {
                        "label": i18next.t('statsPage.Buys'),
                        "data": buys,
                        "backgroundColor":"#ef5350",
                    }
                ]
            },
            "options":{
                "responsive": true,
                "maintainAspectRatio": false,
                "legend": {
                    "display": true,
                },
                "animation": false,
                "tooltips": {

                },
                "hover": {
                    "mode": "nearest",
                    "intersect": true,
                },
                "scales": {
                    "yAxes": [{
                        "ticks": {
                            "fontColor":"rgba(152,166,173,0.5)",
                            beginAtZero: true
                        },
                        "gridLines": {
                            "display": true,
                            "color": "rgba(152,166,173,0.1)",
                            "zeroLineColor": "rgba(152,166,173,0.1)",
                        }
                    }],
                    "xAxes": [{
                        "ticks": {
                            "fontColor":"rgba(152,166,173,0.5)",
                            "maxTicksLimit": 10,
                        },
                        "gridLines": {
                            "display": true,
                            "color": "rgba(152,166,173,0.1)",
                        }
                    }]
                },
                plugins: {
                    labels: false
                }
            }
        });
}

function prepareEarnedVsSpentCharts(sales, element) {
    let labels = []
    let spent = []
    let earned = []

    for (i = 0; i < sales.length; i++) {
        let date
        if(element === "monthlySpentEarnedChart") {
            date = sales[i].month.replace("-", "/")
        } else {
            date = sales[i].date.replace("-", "/").replace("-", "/")
        }

        labels.push(date)

        spent.push(parseFloat((sales[i].totalBoughtCost).toFixed(8)))
        earned.push(parseFloat((sales[i].totalSoldValue).toFixed(8)))
    }

    new Chart(document.getElementById(element),
        {
            "type":"bar",
            "data": {
                "labels": labels,
                "datasets":[
                    {
                        "label": i18next.t('statsPage.Spent'),
                        "data": spent,
                        "backgroundColor":"rgb(75, 192, 192)",
                    },
                    {
                        "label": i18next.t('statsPage.Earned'),
                        "data": earned,
                        "backgroundColor":"#ef5350",
                    }
                ]
            },
            "options":{
                "responsive": true,
                "maintainAspectRatio": false,
                "legend": {
                    "display": true,
                },
                "animation": false,
                "tooltips": {

                },
                "hover": {
                    "mode": "nearest",
                    "intersect": true,
                },
                "scales": {
                    "yAxes": [{
                        "ticks": {
                            "fontColor":"rgba(152,166,173,0.5)",
                            beginAtZero: true,
                            callback: function(value, index, values) {
                                return value.toFixed(8).toString()
                            }
                        },
                        "gridLines": {
                            "display": true,
                            "color": "rgba(152,166,173,0.1)",
                            "zeroLineColor": "rgba(152,166,173,0.1)",
                        }
                    }],
                    "xAxes": [{
                        "ticks": {
                            "fontColor":"rgba(152,166,173,0.5)",
                            "maxTicksLimit": 10,
                        },
                        "gridLines": {
                            "display": true,
                            "color": "rgba(152,166,173,0.1)",
                        }
                    }]
                },
                plugins: {
                    labels: false
                }
            }
        });
}

function prepareProfitCharts(sales, element) {

    let labels = []
    let profits = []


    for (i = 0; i < sales.length; i++) {
        let date
        if(element === "monthlyProfitsChart") {
            date = sales[i].month.replace("-", "/")
        } else {
            date = sales[i].date.replace("-", "/").replace("-", "/")
        }

        labels.push(date)

        profits.push(parseFloat((sales[i].totalProfitCurrency).toFixed(8)))
    }

    new Chart(document.getElementById(element),
        {
            "type":"line",
            "data": {
                "labels": labels,
                "datasets":[
                    {
                        "label": i18next.t('statsPage.Profits'),
                        "data": profits,
                        "fill":false,
                        "borderColor":"rgb(75, 192, 192)",
                        "pointRadius": 3,
                        "lineTension":0.4
                    }
                ]
            },
            "options":{
                "responsive": true,
                "maintainAspectRatio": false,
                "legend": {
                    "display": true,
                },
                "animation": false,
                "tooltips": {
                    "mode": "nearest",
                    "intersect": false,
                },
                "hover": {
                    "mode": "nearest",
                    "intersect": true,
                },
                "scales": {
                    "yAxes": [{
                        "ticks": {
                            "fontColor":"rgba(152,166,173,0.5)",
                            "beginAtZero": true,
                            callback: function(value, index, values) {
                                return value.toFixed(8).toString()
                            }
                        },
                        "gridLines": {
                            "display": true,
                            "color": "rgba(152,166,173,0.1)",
                            "zeroLineColor": "rgba(152,166,173,0.1)",
                        }
                    }],
                    "xAxes": [{
                        "ticks": {
                            "fontColor":"rgba(152,166,173,0.5)",
                            "maxTicksLimit": 10,

                        },
                        "gridLines": {
                            "display": true,
                            "color": "rgba(152,166,173,0.1)",
                        }
                    }]
                }
            }
        });

}

